<?php
	include("config.php");
	include("func.php");

	function stitchTiles($lat, $lon, $zoom, $px, $py, $w, $h, $baseurl) {
		// get tile number _including_ decimal places
		$x = getTileX($lon, $zoom);
		$y = getTileY($lat, $zoom);

		// which tiles do we have to stitch?
		$xmin = (int)($x + ($px - 2.5)*$w / 256);
		$ymin = (int)($y + ($py - 2.5)*$h / 256);
		$xmax = (int)($x + ($px - 1.5)*$w / 256);
		$ymax = (int)($y + ($py - 1.5)*$h / 256);

		// area for stiching
		$bw = ($xmax - $xmin + 1) * 256;
		$bh = ($ymax - $ymin + 1) * 256;

		// the center of what we really want
		$cx = ($x - $xmin) * 256 + ($px - 2) * $w;
		$cy = ($y - $ymin) * 256 + ($py - 2) * $h;

		// true color needed to avoid color problems with the reduced palettes
		$back = ImageCreateTrueColor($bw, $bh);
		$white = ImageColorAllocate($back, 255, 255, 255);
		ImageFill($back, 0, 0, $white);
		ImageColorTransparent($back, $white);

		for($i = $xmin; $i <= $xmax; $i++) {
			for($j = $ymin; $j <= $ymax; $j++) {
				$src = ImageCreateFromPNG("$baseurl/$zoom/$i/$j.png");
				ImageCopy($back, $src, ($i - $xmin)*256, ($j - $ymin)*256, 0, 0, 256, 256);
				ImageDestroy($src);
			}
		}
		$base = ImageCreateTrueColor($w, $h);
		$white = ImageColorAllocate($base, 255, 255, 255);
		ImageFill($base, 0, 0, $white);
		ImageColorTransparent($base, $white);
		ImageCopy($base, $back, 0, 0, $cx - $w/2, $cy - $h/2, $w, $h);
		ImageDestroy($back);
		return $base;
	}

	function getTileBase($lat, $lon, $zoom, $px, $py, $w, $h) {
		return stitchTiles($lat, $lon, $zoom, $px, $py, $w, $h, "http://tile.openstreetmap.org");
	}

	function getTahBase($lat, $lon, $zoom, $px, $py, $w, $h) {
		return stitchTiles($lat, $lon, $zoom, $px, $py, $w, $h, "http://tah.openstreetmap.org/Tiles/tile");
	}

	function getCycleBase($lat, $lon, $zoom, $px, $py, $w, $h) {
		return stitchTiles($lat, $lon, $zoom, $px, $py, $w, $h, "http://andy.sandbox.cloudmade.com/tiles/cycle");
	}

	function getLandsatBase($lat, $lon, $zoom, $px, $py, $w, $h) {
		$dlon = 360/pow(2, $zoom);
		$dlat = 360 * cos($lat * pi() / 180)/pow(2, $zoom);

		$north = $lat + (2 - $py + 0.5)*$h * $dlat / 256;
		$south = $lat + (2 - $py - 0.5)*$h * $dlat / 256;
		$east = $lon + ($px - 1.5)*$w * $dlon / 256;
		$west = $lon + ($px - 2.5)*$w * $dlon / 256;

		$maxlat = ceil($north * 32) / 32;
		$minlat = floor($south *32) / 32;
		$maxlon = ceil($east * 32) / 32;
		$minlon = floor($west * 32) / 32;
		$lontiles = ($maxlon - $minlon)* 32;
		$lattiles = ($maxlat - $minlat) * 32;

		$back = ImageCreateTrueColor($lontiles * 512, $lattiles * 512);
		$black = ImageColorAllocate($back, 0, 0, 0);
		ImageFilledRectangle($back, 0, 0, $lontiles * 512, $lattiles * 512, $blue);

		for($i = 0; $i < $lontiles; $i++) {
			for($j = 0; $j < $lattiles; $j++) {
				$westend = $minlon + $i * 0.03125;
				$eastend = $minlon + ($i+1) * 0.03125;
				$southend = $minlat + $j * 0.03125;
				$northend = $minlat + ($j+1) * 0.03125;
				$bbox = "bbox=$westend,$southend,$eastend,$northend";
				$url = "http://wms.jpl.nasa.gov/wms.cgi?request=GetMap&layers=global_mosaic&srs=EPSG:4326&format=image/jpeg&styles=&width=512&height=512&$bbox";
				@$src = ImageCreateFromJPEG($url);
				if($src) {
					ImageCopy($back, $src, $i*512, ($lattiles - $j - 1)*512, 0, 0, 512, 512);
					ImageDestroy($src);
				} else {
					ImageFilledRectangle($back, $i*512, $j*512, ($i + 1)*512, ($j + 1)*512, $black);
				}
			}
		}
		$l = 32*512*($west - $minlon);
		$r = 32*512*($east - $minlon);
		$t = 32*512*($maxlat - $north);
		$b = 32*512*($maxlat - $south);
		$base = ImageCreateTrueColor($w, $h);
		ImageCopyResized($base, $back, 0, 0, $l, $t, $w, $h, $r - $l, $b - $t);
		ImageDestroy($back);
		return $base;
	}

	function getMaplintOverlay($lat, $lon, $zoom, $px, $py, $w, $h) {
		return stitchTiles($lat, $lon, $zoom, $px, $py, $w, $h, "http://tah.openstreetmap.org/Tiles/maplint");
	}

	function getBugOverlay($lat, $lon, $zoom, $px, $py, $w, $h) {
		$pad = 25;

		$dlon = 360/pow(2, $zoom);
		$dlat = 360 * cos($lat * pi() / 180)/pow(2, $zoom);

		$north = $lat + ((2 - $py + 0.5)*$h + $pad)*$dlat / 256;
		$south = $lat + ((2 - $py - 0.5)*$h - $pad)*$dlat / 256;
		$east  = $lon + (($px - 2 + 0.5)*$w + $pad)*$dlon / 256;
		$west  = $lon + (($px - 2 - 0.5)*$w - $pad)*$dlon / 256;

		$back = ImageCreateTrueColor($w + 4*$pad, $h + 4*$pad);
		$white = ImageColorAllocate($back, 255, 255, 255);
		$red = ImageColorAllocate($back, 255, 0, 0);
		$blue = ImageColorAllocate($back, 100, 100, 255);
		$black = ImageColorAllocate($back, 0, 0, 0);
		ImageFill($back, 0, 0, $white);
		ImageColorTransparent($back, $white);

		$bugs = getBugs($north, $south, $west, $east);
		while(list($key, $bug) = each($bugs)) {
			if($bug[4] == 0) {
				$col = $red;
			} else {
				$col = $blue;
			}
			$x = ($bug[1] - $west)*256/$dlon;
			$y = ($north - $bug[2])*256/$dlat;
			ImageFilledEllipse($back, $pad + $x, $pad + $y, 9, 9, $black);
			ImageFilledEllipse($back, $pad + $x, $pad + $y, 7, 7, $col);
		}
		$overlay = ImageCreateTrueColor($w, $h);
		$white = ImageColorAllocate($overlay, 255, 255, 255);
		ImageFill($overlay, 0, 0, $white);
		ImageColorTransparent($overlay, $white);
		ImageCopy($overlay, $back, 0, 0, 2*$pad, 2*$pad, $w, $h);
		ImageDestroy($back);
		return $overlay;
	}

	function getLinkOverlay($lat, $lon, $zoom, $px, $py, $w, $h) {
		$pad = 10;
		$dlon = 360/pow(2, $zoom);
		$dlat = 360 * cos($lat * pi() / 180)/pow(2, $zoom);

		$north = $lat + ((2 - $py + 0.5)*$h + $pad)*$dlat / 256;
		$south = $lat + ((2 - $py - 0.5)*$h - $pad)*$dlat / 256;
		$east = $lon + (($px - 1.5)*$w + $pad)*$dlon / 256;
		$west = $lon + (($px - 2.5)*$w - $pad)*$dlon / 256;

		$back = ImageCreateTrueColor($w + 4*$pad, $h + 4*$pad);
		$white = ImageColorAllocate($back, 255, 255, 255);
		$green = ImageColorAllocate($back, 0, 255, 0);
		$black = ImageColorAllocate($back, 0, 0, 0);
		ImageFill($back, 0, 0, $white);
		ImageColorTransparent($back, $white);

		$dom = new DOMDocument;
		if(@$dom->load("http://www.informationfreeway.org/api/0.5/node[website|url|link=*][bbox={$west},{$south},{$east},{$north}]")) {
			$xpath = new DOMXPath($dom);

			$nodes = $xpath->query('/osm/node');
			foreach($nodes as $node) {
				$lat =  getAttributeFromElement($node, "lat");
				$lon =  getAttributeFromElement($node, "lon");
				$x = ($lon - $west)*256/$dlon;
				$y = ($north - $lat)*256/$dlat;
				ImageFilledEllipse($back, $pad + $x, $pad + $y, 9, 9, $black);
				ImageFilledEllipse($back, $pad + $x, $pad + $y, 7, 7, $green);
			}
		}
		$overlay = ImageCreateTrueColor($w, $h);
		$white = ImageColorAllocate($overlay, 255, 255, 255);
		ImageFill($overlay, 0, 0, $white);
		ImageColorTransparent($overlay, $white);
		ImageCopy($overlay, $back, 0, 0, 2*$pad, 2*$pad, $w, $h);
		ImageDestroy($back);
		return $overlay;
	}

	function getGPXOverlay($lat, $lon, $zoom, $px, $py, $w, $h, $filename, $trackcolor) {
		global $uploaddir;
		$dlon = 360/pow(2, $zoom);
		$dlat = 360 * cos($lat * pi() / 180)/pow(2, $zoom);

		$north = $lat + (2 - $py + 0.5)*$h*$dlat / 256;
		$south = $lat + (2 - $py - 0.5)*$h*$dlat / 256;
		$east = $lon + ($px - 1.5)*$w*$dlon / 256;
		$west = $lon + ($px - 2.5)*$w*$dlon / 256;

		$back = ImageCreateTrueColor($w, $h);
		$white = ImageColorAllocate($back, 255, 255, 255);
		$colorArray = array();
		if(preg_match('/[0-9a-fA-F]{6}/', $trackcolor)) {
			$colorArray = sscanf($trackcolor, '%2x%2x%2x');
		} else {
			$colorArray = array(0, 255, 0);
		}
		$green = ImageColorAllocate($back, $colorArray[0],$colorArray[1], $colorArray[2]);
		ImageFill($back, 0, 0, $white);
		ImageColorTransparent($back, $white);

		$dom = new DOMDocument;
		if($dom->load("$uploaddir$filename")) {
			$gpx_nodes = $dom->getElementsByTagName("gpx");
			$namespace = "";
			foreach($gpx_nodes as $node) {
				$namespace = $node->getAttribute("xmlns");
			}
			$xpath = new DOMXPath($dom);
			//$xpath->registerNameSpace("g", "http://www.topografix.com/GPX/1/1");
			$xpath->registerNameSpace("g", $namespace);
			$nodes = $xpath->query('//g:trkpt');
			$lastX = 0;
			$lastY = 0;
			foreach($nodes as $node) {
				$lat = getAttributeFromElement($node, "lat");
				$lon = getAttributeFromElement($node, "lon");
				$x = ($lon - $west)*256/$dlon;
				$y = ($north - $lat)*256/$dlat;
				if($lastX != 0 && $lastY != 0) {
					ImageLine($back, $lastX, $lastY, $x, $y, $green);
				} else {
					ImageSetPixel($back, $x, $y, $green);
				}
				$lastX = $x;
				$lastY = $y;
			}
		}
		return $back;
	}

	Header("Content-type: image/png");

	$lat=$_REQUEST["lat"];
	$lon=$_REQUEST["lon"];
	$zoom=$_REQUEST["zoom"];

	$px=$_REQUEST["px"];
	$py=$_REQUEST["py"];
	$w=$_REQUEST["width"];
	$h=$_REQUEST["height"];
	$layer=$_REQUEST["layer"];
	$gpx=$_REQUEST["gpx"];
	$trackcolor = getValueFromRequest("trackcolor", "00ff00");
	$layerArray = parseLayer($layer);

	if($layerArray["base"] == "tile") {
		$base = getTileBase($lat, $lon, $zoom, $px, $py, $w, $h);
	} else if($layerArray["base"] == "tah") {
		$base = getTahBase($lat, $lon, $zoom, $px, $py, $w, $h);
	} else if($layerArray["base"] == "cycle") {
		$base = getCycleBase($lat, $lon, $zoom, $px, $py, $w, $h);
	} else if($layerArray["base"] == "landsat") {
		$base = getLandsatBase($lat, $lon, $zoom, $px, $py, $w, $h);
	} else {
		$base = ImageCreateTrueColor($w, $h);
		$black = ImageColorAllocate($base, 0, 0, 0);
		ImageFill($base, 0, 0, $black);
	}

	if($layerArray["maplint"]) {
		$overlay = getMaplintOverlay($lat, $lon, $zoom, $px, $py, $w, $h);
		ImageCopyMerge($base, $overlay, 0, 0, 0, 0, $w, $h, 66);
		ImageDestroy($overlay);
	}
	if($layerArray["osb"]) {
		$overlay = getBugOverlay($lat, $lon, $zoom, $px, $py, $w, $h);
		ImageCopy($base, $overlay, 0, 0, 0, 0, $w, $h);
		ImageDestroy($overlay);
	}
	if($layerArray["links"]) {
		$overlay = getLinkOverlay($lat, $lon, $zoom, $px, $py, $w, $h);
		ImageCopy($base, $overlay, 0, 0, 0, 0, $w, $h);
		ImageDestroy($overlay);
	}
	if($gpx != "") {
		$overlay = getGPXOverlay($lat, $lon, $zoom, $px, $py, $w, $h, $gpx, $trackcolor);
		ImageCopy($base, $overlay, 0, 0, 0, 0, $w, $h);
		ImageDestroy($overlay);
	}

	ImagePng($base);
	ImageDestroy($base);
?>
